Player = Object:extend()

function Player:new(sprite,x,y,vel,stage)
  self.sprite = love.graphics.newImage(sprite)
  self.width, self.height = self.sprite:getDimensions()

  -- pointer to stage
  self.stage = stage

  -- physics
  self.vel = vel

  self.body = love.physics.newBody(stage.world,x,y,'dynamic')
  self.shape = love.physics.newRectangleShape(self.width,self.height)
  self.fixture = love.physics.newFixture(self.body,self.shape)

  -- prevents body from rotating
  self.body:setFixedRotation(true)

  self.fixture:setFriction(0)

  -- add self to stage
  stage:addObject(self,'mid')
end

function Player:draw()
  local offset_x,offset_y = self.sprite:getWidth()/2,self.sprite:getHeight()/2
  local x,y = self.body:getX(),self.body:getY()
  local angle = self.body:getAngle()

  love.graphics.draw(self.sprite,x,y,angle,1,1,offset_x,offset_y)
end

function Player:update(dt)
  self:movement(dt)
end

function Player:setControl(control)
  self.control = control
end

function Player:movement(dt)
  local x_vel, y_vel = self.body:getLinearVelocity()

  local down_left = love.keyboard.isDown(self.control.left)
  local down_right = love.keyboard.isDown(self.control.right)
  local down_jump = love.keyboard.isDown(self.control.jump)

  local factor_x = 0

  -- ==================================================

  local x = self.body:getX() - self.sprite:getWidth()/2
  local y = self.body:getY()
  current_side = 'bottom_left'
  self.stage.world:rayCast(x,y,x,love.graphics.getHeight(),raycastCallback)

  x = self.body:getX() + self.sprite:getWidth()/2
  current_side = 'bottom_right'
  self.stage.world:rayCast(x,y,x,love.graphics.getHeight(),raycastCallback)

  local grounded = self:isTouching('bottom')

  -- x = self.body:getX()
  -- y = self.body:getY() - self.sprite:getHeight()/2
  -- current_side = 'left_upper'
  -- self.stage.world:rayCast(x,y,0,y,raycastCallback)

  -- y = self.body:getY() + self.sprite:getHeight()/2
  -- current_side = 'left_lower'
  -- self.stage.world:rayCast(x,y,0,y,raycastCallback)

  -- local touching_left = self:isTouching('left')

  -- y = self.body:getY() - self.sprite:getHeight()/2
  -- current_side = 'right_upper'
  -- self.stage.world:rayCast(x,y,love.graphics.getWidth(),y,raycastCallback)

  -- y = self.body:getY() + self.sprite:getHeight()/2
  -- current_side = 'right_lower'
  -- self.stage.world:rayCast(x,y,love.graphics.getWidth(),y,raycastCallback)

  -- local touching_right = self:isTouching('right')

  -- ===================================================

  if down_left and down_right then
    factor_x = 0
  else
    if down_left then
      factor_x = -1
    elseif down_right then
      factor_x = 1
    end
  end

  self.body:setLinearVelocity(factor_x*self.vel, y_vel)

  if down_jump and grounded then
    self.body:applyLinearImpulse(0,-800)
  end

  objects_hit.bottom_left = {}
  objects_hit.bottom_right = {}
  objects_hit.left_upper = {}
  objects_hit.left_lower = {}
  objects_hit.right_upper = {}
  objects_hit.right_lower = {}
end

function raycastCallback(fixture,x,y,xn,yn,fraction)
  table.insert(objects_hit[current_side],{fixture=fixture,x=x,y=y,fraction=fraction})

  return 1
end

function Player:isTouching(side)
  local x = 0
  local y = 0

  if side == 'bottom' then
    local closest = objects_hit.bottom_left[1] or objects_hit.bottom_right[1]

    if closest then
      for _,edge in pairs({'left','right'}) do
        for __,object in pairs(objects_hit[side..'_'..edge]) do
          if object.fraction <= closest.fraction then
            closest = object

            if edge == 'left' then
              x = self.body:getX() - self.sprite:getWidth()/2
            elseif edge == 'right' then
              x = self.body:getX() + self.sprite:getWidth()/2
            end
          end
        end
      end

      local y = self.body:getY() + self.sprite:getHeight()/2
      local distance = math.sqrt(math.pow(x-closest.x,2)+math.pow(y-closest.y,2))

      return distance < 1
    else
      return false
    end

  -- elseif side == 'left' then
  --   local closest = objects_hit.left_upper[1] or objects_hit.left_lower[1]

  --   if closest then
  --     for _,edge in pairs({'upper','lower'}) do
  --       for __,object in pairs(objects_hit[side..'_'..edge]) do
  --         if object.fraction <= closest.fraction then
  --           closest = object

  --           if edge == 'upper' then
  --             y = self.body:getY() - self.sprite:getHeight()/2
  --           elseif edge == 'lower' then
  --             y = self.body:getY() + self.sprite:getHeight()/2
  --           end
  --         end
  --       end
  --     end

  --     local x = self.body:getX() - self.sprite:getWidth()/2
  --     local distance = math.sqrt(math.pow(x-closest.x,2)+math.pow(y-closest.y,2))

  --     return distance < 1
  --   else
  --     return false
  --   end

  -- elseif side == 'right' then
  --   local closest = objects_hit.right_upper[1] or objects_hit.right_lower[1]

  --   if closest then
  --     for _,edge in pairs({'upper','lower'}) do
  --       for __,object in pairs(objects_hit[side..'_'..edge]) do
  --         if object.fraction <= closest.fraction then
  --           closest = object

  --           if edge == 'upper' then
  --             y = self.body:getY() - self.sprite:getHeight()/2
  --           elseif edge == 'lower' then
  --             y = self.body:getY() + self.sprite:getHeight()/2
  --           end
  --         end
  --       end
  --     end

  --     local x = self.body:getX() + self.sprite:getWidth()/2
  --     local distance = math.sqrt(math.pow(x-closest.x,2)+math.pow(y-closest.y,2))

  --     return distance < 1
  --   else
  --     return false
  --   end
  end
end

-- for detecting collisions
objects_hit = {}
current_side = nil

objects_hit.bottom_left = {}
objects_hit.bottom_right = {}

objects_hit.left_upper = {}
objects_hit.left_lower = {}

objects_hit.right_upper = {}
objects_hit.right_lower = {}